{% extends "admin/base_site.html" %}
{% load i18n static filer_admin_tags %}

{% block extrastyle %}
    {{ block.super }}
    <link rel="stylesheet" type="text/css" href="{% static 'filer/css/admin_filer.css' %}">
    {% icon_css_library %}
{% endblock %}

{% block extrahead %}
    {{ block.super }}

    {# upload stuff #}
    {{ media.js }}
    <script src="{% static 'admin/js/admin/RelatedObjectLookups.js' %}"></script>
    <script src="{% static 'filer/js/dist/filer-base.bundle.js' %}"></script>
{% endblock %}
