# Generated by Django 1.11.7 on 2017-12-20 16:35
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0008_auto_20171117_1313'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='_file_size',
            field=models.BigIntegerField(blank=True, null=True, verbose_name='file size'),
        ),
    ]
