# Generated by Django 1.9.13 on 2017-11-17 13:13
from django.db import migrations, models

import filer.models.filemodels


class Migration(migrations.Migration):

    dependencies = [
        ('filer', '0007_auto_20161016_1055'),
    ]

    operations = [
        migrations.AlterField(
            model_name='file',
            name='is_public',
            field=models.BooleanField(default=filer.models.filemodels.is_public_default, help_text='Disable any permission checking for this file. File will be publicly accessible to anyone.', verbose_name='Permissions disabled'),
        ),
    ]
